"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MysqlQuery = void 0;
const moment_timezone_1 = __importDefault(require("moment-timezone"));
const shared_1 = require("@cubejs-backend/shared");
const BaseQuery_1 = require("./BaseQuery");
const BaseFilter_1 = require("./BaseFilter");
const UserError_1 = require("../compiler/UserError");
const GRANULARITY_TO_INTERVAL = {
    day: (date) => `DATE_FORMAT(${date}, '%Y-%m-%dT00:00:00.000')`,
    week: (date) => `DATE_FORMAT(DATE_ADD('1900-01-01', INTERVAL TIMESTAMPDIFF(WEEK, '1900-01-01', ${date}) WEEK), '%Y-%m-%dT00:00:00.000')`,
    hour: (date) => `DATE_FORMAT(${date}, '%Y-%m-%dT%H:00:00.000')`,
    minute: (date) => `DATE_FORMAT(${date}, '%Y-%m-%dT%H:%i:00.000')`,
    second: (date) => `DATE_FORMAT(${date}, '%Y-%m-%dT%H:%i:%S.000')`,
    month: (date) => `DATE_FORMAT(${date}, '%Y-%m-01T00:00:00.000')`,
    quarter: (date) => `DATE_ADD('1900-01-01', INTERVAL TIMESTAMPDIFF(QUARTER, '1900-01-01', ${date}) QUARTER)`,
    year: (date) => `DATE_FORMAT(${date}, '%Y-01-01T00:00:00.000')`
};
class MysqlFilter extends BaseFilter_1.BaseFilter {
    likeIgnoreCase(column, not, param, type) {
        const p = (!type || type === 'contains' || type === 'ends') ? '%' : '';
        const s = (!type || type === 'contains' || type === 'starts') ? '%' : '';
        return `${column}${not ? ' NOT' : ''} LIKE CONCAT('${p}', ${this.allocateParam(param)}, '${s}')`;
    }
}
class MysqlQuery extends BaseQuery_1.BaseQuery {
    newFilter(filter) {
        return new MysqlFilter(this, filter);
    }
    castToString(sql) {
        return `CAST(${sql} as CHAR)`;
    }
    convertTz(field) {
        return `CONVERT_TZ(${field}, @@session.time_zone, '${(0, moment_timezone_1.default)().tz(this.timezone).format('Z')}')`;
    }
    timeStampCast(value) {
        return `TIMESTAMP(convert_tz(${value}, '+00:00', @@session.time_zone))`;
    }
    timestampFormat() {
        return moment_timezone_1.default.HTML5_FMT.DATETIME_LOCAL_MS;
    }
    dateTimeCast(value) {
        return `TIMESTAMP(${value})`;
    }
    subtractInterval(date, interval) {
        return `DATE_SUB(${date}, INTERVAL ${this.formatInterval(interval)})`;
    }
    addInterval(date, interval) {
        return `DATE_ADD(${date}, INTERVAL ${this.formatInterval(interval)})`;
    }
    timeGroupedColumn(granularity, dimension) {
        return `CAST(${GRANULARITY_TO_INTERVAL[granularity](dimension)} AS DATETIME)`;
    }
    /**
     * Returns sql for source expression floored to timestamps aligned with
     * intervals relative to origin timestamp point.
     */
    dateBin(interval, source, origin) {
        const intervalFormatted = this.formatInterval(interval);
        const timeUnit = this.isIntervalYM(interval) ? 'MONTH' : 'SECOND';
        return `TIMESTAMPADD(${timeUnit},
        FLOOR(
          TIMESTAMPDIFF(${timeUnit}, ${this.dateTimeCast(`'${origin}'`)}, ${source}) /
          TIMESTAMPDIFF(${timeUnit}, '1970-01-01 00:00:00', '1970-01-01 00:00:00' + INTERVAL ${intervalFormatted})
        ) * TIMESTAMPDIFF(${timeUnit}, '1970-01-01 00:00:00', '1970-01-01 00:00:00' + INTERVAL ${intervalFormatted}),
        ${this.dateTimeCast(`'${origin}'`)}
    )`;
    }
    isIntervalYM(interval) {
        return /(year|month|quarter)/i.test(interval);
    }
    /**
     * The input interval with (possible) plural units, like "2 years", "3 months", "4 weeks", "5 days"...
     * will be converted to MYSQL dialect.
     * @see https://dev.mysql.com/doc/refman/8.4/en/expressions.html#temporal-intervals
     */
    formatInterval(interval) {
        const intervalParsed = (0, shared_1.parseSqlInterval)(interval);
        const intKeys = Object.keys(intervalParsed).length;
        if (intervalParsed.year && intKeys === 1) {
            return `${intervalParsed.year} YEAR`;
        }
        else if (intervalParsed.year && intervalParsed.month && intKeys === 2) {
            return `'${intervalParsed.year}-${intervalParsed.month}' YEAR_MONTH`;
        }
        else if (intervalParsed.quarter && intKeys === 1) {
            return `${intervalParsed.quarter} QUARTER`;
        }
        else if (intervalParsed.month && intKeys === 1) {
            return `${intervalParsed.month} MONTH`;
        }
        else if (intervalParsed.week && intKeys === 1) {
            return `${intervalParsed.week} WEEK`;
        }
        else if (intervalParsed.day && intKeys === 1) {
            return `${intervalParsed.day} DAY`;
        }
        else if (intervalParsed.day && intervalParsed.hour && intKeys === 2) {
            return `'${intervalParsed.day} ${intervalParsed.hour}' DAY_HOUR`;
        }
        else if (intervalParsed.day && intervalParsed.hour && intervalParsed.minute && intKeys === 3) {
            return `'${intervalParsed.day} ${intervalParsed.hour}:${intervalParsed.minute}' DAY_MINUTE`;
        }
        else if (intervalParsed.day && intervalParsed.hour && intervalParsed.minute && intervalParsed.second && intKeys === 4) {
            return `'${intervalParsed.day} ${intervalParsed.hour}:${intervalParsed.minute}:${intervalParsed.second}' DAY_SECOND`;
        }
        else if (intervalParsed.hour && intervalParsed.minute && intKeys === 2) {
            return `'${intervalParsed.hour}:${intervalParsed.minute}' HOUR_MINUTE`;
        }
        else if (intervalParsed.hour && intervalParsed.minute && intervalParsed.second && intKeys === 3) {
            return `'${intervalParsed.hour}:${intervalParsed.minute}:${intervalParsed.second}' HOUR_SECOND`;
        }
        else if (intervalParsed.minute && intervalParsed.second && intKeys === 2) {
            return `'${intervalParsed.minute}:${intervalParsed.second}' MINUTE_SECOND`;
        }
        // No need to support microseconds.
        throw new Error(`Cannot transform interval expression "${interval}" to MySQL dialect`);
    }
    escapeColumnName(name) {
        return `\`${name}\``;
    }
    seriesSql(timeDimension) {
        const values = timeDimension.timeSeries().map(([from, to]) => `select '${from}' f, '${to}' t`).join(' UNION ALL ');
        return `SELECT TIMESTAMP(dates.f) date_from, TIMESTAMP(dates.t) date_to FROM (${values}) AS dates`;
    }
    concatStringsSql(strings) {
        return `CONCAT(${strings.join(', ')})`;
    }
    unixTimestampSql() {
        return 'UNIX_TIMESTAMP()';
    }
    wrapSegmentForDimensionSelect(sql) {
        return `IF(${sql}, 1, 0)`;
    }
    preAggregationTableName(cube, preAggregationName, skipSchema) {
        const name = super.preAggregationTableName(cube, preAggregationName, skipSchema);
        if (name.length > 64) {
            throw new UserError_1.UserError(`MySQL can not work with table names that longer than 64 symbols. Consider using the 'sqlAlias' attribute in your cube and in your pre-aggregation definition for ${name}.`);
        }
        return name;
    }
    sqlTemplates() {
        const templates = super.sqlTemplates();
        templates.quotes.identifiers = '`';
        templates.quotes.escape = '\\`';
        // NOTE: this template contains a comma; two order expressions are being generated
        templates.expressions.sort = '{{ expr }} IS NULL {% if nulls_first %}DESC{% else %}ASC{% endif %}, {{ expr }} {% if asc %}ASC{% else %}DESC{% endif %}';
        delete templates.expressions.ilike;
        templates.types.string = 'VARCHAR';
        templates.types.boolean = 'TINYINT';
        templates.types.timestamp = 'DATETIME';
        delete templates.types.interval;
        templates.types.binary = 'BLOB';
        return templates;
    }
}
exports.MysqlQuery = MysqlQuery;
//# sourceMappingURL=MysqlQuery.js.map